#include <malloc.h>
#include <string.h>

#include "ihx.h"

using namespace ihx;

BuffElem::BuffElem() {
	mem = NULL;
	size = 0;
	address = 0;
}

BuffElem::BuffElem(const BuffElem &b) {
	size = b.size;
	address = b.address;
	mem = (unsigned char*)malloc(size);
	memcpy(mem, b.mem, size);
}

BuffElem & BuffElem::operator = (const BuffElem &b) {
	Clear();
	size = b.size;
	address = b.address;
	mem = (unsigned char*)malloc(size);
	memcpy(mem, b.mem, size);
	return *this;
}

BuffElem::BuffElem(unsigned int init_size) {
	address = 0;
	size = init_size;
	mem = (unsigned char*)malloc(size);
}

BuffElem::BuffElem(unsigned int a, unsigned char *b, unsigned int s) {
	address = a;
	size = s;
	mem = (unsigned char*)malloc(size);
	memcpy(mem, b, size);
}

BuffElem::~BuffElem() {
	Clear();
}

void BuffElem::Clear() {
	if (mem) free(mem);
	size = 0;
	mem = NULL;
}

bool BuffElem::TouchesLeft(const BuffElem &e) const {
	return e.address + e.size == address;
}

bool BuffElem::TouchesRight(const BuffElem &e) const {
	return address + size == e.address;
}

bool BuffElem::Touches(const BuffElem &e) const {
	return TouchesLeft(e) || TouchesRight(e);
}

bool BuffElem::Overrides(const BuffElem &e) const {
	return e.address >= address &&
		(e.address + e.size) <= (address + size);
}

bool BuffElem::CoversLeft(const BuffElem &e) const {
	return address >= e.address &&
		address < (e.address + e.size);
}

bool BuffElem::CoversRight(const BuffElem &e) const {
	return (address + size - 1) >= e.address &&
		(address + size) <= (e.address + e.size);
}

bool BuffElem::Covers(const BuffElem &e) const {
	return CoversLeft(e) || CoversRight(e);
}

unsigned char *BuffElem::GetBuffer() {
	return mem;
}

const unsigned char *BuffElem::GetBuffer() const {
	return mem;
}

unsigned int BuffElem::GetSize() const {
	return size;
}

unsigned char *BuffElem::Alloc(unsigned int size) {
	Clear();
	return mem = (unsigned char *)malloc(size);
}

void BuffElem::Merge(const BuffElem &e) {
	unsigned int ea = e.address + e.size > address + size ?
				e.address + e.size : address + size;

	if (e.address < address) {
		unsigned int ts;
		mem = (unsigned char *)realloc(mem, ts = (ea - e.address));
		memmove(mem + address - e.address, mem, size);
		memcpy(mem, e.mem, e.size);
		address = e.address;
		size = ts;
	} else {
		mem = (unsigned char *)realloc(mem, size = (ea - address));
		memcpy(mem + e.address - address, e.mem, e.size);
	}
}

void BuffElem::Dump(FILE *f) const {
	fprintf(f, "0x%04X-0x%04X: ", address, address + size - 1);
	for (unsigned int i = 0; i < size; ++i)
		fprintf(f, "%02X ", mem[i]);
	fprintf(f, "\n");
}
