#ifndef _IHX_H_
#define _IHX_H_

#include <vector>
#include <stdio.h>

namespace ihx {

class BuffElem {
	public:
		unsigned int address;

		BuffElem();
		BuffElem(const BuffElem &c);
		BuffElem(unsigned int init_size);
		BuffElem(unsigned int address, unsigned char *buf, unsigned int size);
		~BuffElem();

		unsigned char *Alloc(unsigned int size);
		unsigned char *GetBuffer();
		const unsigned char *GetBuffer() const;
		unsigned int GetSize() const;
		void Clear();

		void Merge(const BuffElem &e);

		bool TouchesLeft(const BuffElem &e) const;
		bool TouchesRight(const BuffElem &e) const;
		bool Touches(const BuffElem &e) const;
		bool Overrides(const BuffElem &e) const;
		bool CoversLeft(const BuffElem &e) const;
		bool CoversRight(const BuffElem &e) const;
		bool Covers(const BuffElem &e) const;

		void Dump(FILE *f = stdout) const;

		BuffElem & operator = (const BuffElem &e);

	private:
		unsigned char *mem;
		unsigned int size;
};

class Buffer : public std::vector<BuffElem> {
	public:
		typedef enum {
			ERR_OK = 0,
			ERR_OPEN,
			ERR_FILEFORMAT,
			ERR_CHECKSUM,
			ERR_UNKNOWNREC
		} Error;

		unsigned long eip;

		void AddElement(const BuffElem &el);
		Error ReadIHX(const char *path);
		Error ReadBIN(const char *path, unsigned int offset = 0U);
		Error WriteBIN(const char *path) const;
		unsigned int CalcSize() const;
		void * GetBINBlock(unsigned int size, unsigned int offset = 0U, unsigned char fill = 0x00) const;
		static void FreeBINBlock(void *m);
		unsigned int LastAddress() const;

		static const char * GetErrorString(Error e);
};

} // namespace ihx

#endif
